// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <gsl/gsl_multimin.h>

double get_tail_error(struct fitvars *config)
{
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
double Nc=0.0;
double Nv=0.0;
double mue=0.0;
double muh=0.0;
double pl=0.0;
double nr=0.0;

Ee=fabs(read_value("dos.inp",0,12));
Eh=fabs(read_value("dos.inp",0,14));

mue=fabs(read_value("dos.inp",0,18));
muh=fabs(read_value("dos.inp",0,20));

if (Ee<10e-3) ret+=100.0;
if (Ee>150e-3) ret+=100.0;

if (Eh<10e-3) ret+=100.0;
if (Eh>150e-3) ret+=100.0;

if (muh>mue) ret+=100.0;

Nc=fabs(read_value("dos.inp",0,64));
Nv=fabs(read_value("dos.inp",0,66));
if (Nc<1e24) ret+=100.0;
if (Nv<1e24) ret+=100.0;

pl=fabs(read_value("blom_device.inp",0,11));
nr=fabs(read_value("blom_device.inp",0,15));
if (pl>Nv) ret+=100.0;
if (nr>Nc) ret+=100.0;
return ret;
}

double get_charge_light_ss_error(struct fitvars *config)
{
printf("get ce light ss error\n");
double error=0.0;
double max=0.0;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/celight.dat");
strcpy(sim_name,"./sim/jvlight/charge.dat");
strcpy(out_name,"./ce_light_error.dat");

struct istruct exp;
struct istruct sim;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);


inter_log_y(&sim);
inter_log_y(&exp);

double start=exp.x[1];
if (sim.x[0]>start) start=sim.x[0];
start=0.3;
double stop=0.6;

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);


FILE *ferror;
ferror=fopen(out_name,"w");


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

fclose(ferror);

inter_free(&exp);
inter_free(&sim);

return error;
}


double get_iv_error_light(struct fitvars *config)
{
double error=0.0;
struct istruct exp;
struct istruct sim;

char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/jvlight/jv.dat");
strcpy(sim_name,"./sim/jvlight/ivexternal.dat");
strcpy(out_name,"./ivlight_error.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);

//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");

double start=0.3;
double stop=0.7;

if (exp.x[0]>start) start=exp.x[0];
if (sim.x[0]>start) start=sim.x[0];

if (exp.x[exp.len-1]<stop) stop=exp.x[exp.len-1];
if (sim.x[sim.len-1]<stop) stop=sim.x[sim.len-1];

//inter_chop(&exp,start,stop);
//inter_chop(&sim,start,stop);

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);

double max=0.0;

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->ivlight_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_ss_error(struct fitvars *config)
{
printf("get ce ss error\n");
double error=0.0;
double max=0.0;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/cedata.dat");
strcpy(sim_name,"./sim/jv/charge.dat");
strcpy(out_name,"./ce_error.dat");

struct istruct exp;
struct istruct sim;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);


inter_log_y(&sim);
inter_log_y(&exp);

double start=0.4;
if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];

double stop=0.6;
if (sim.x[sim.len-1]<stop) stop=sim.x[sim.len-1];
if (exp.x[exp.len-1]<stop) stop=exp.x[exp.len-1];

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);



max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



FILE *ferror;
ferror=fopen(out_name,"w");


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

fclose(ferror);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_error(struct fitvars *config)
{
printf("get iv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct sim;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/cedata.dat");
strcpy(sim_name,"./sim/ce/realce.dat");
strcpy(out_name,"./ce_error.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");
//getchar();
//inter_dump(&sim);

inter_log_y(&sim);
inter_log_y(&exp);

double min=exp.data[0];
inter_sub(&sim,min);
inter_sub(&exp,min);


max=inter_norm_to_one(&exp);
inter_rescale(&sim,1.0, 1.0/max);

double start=exp.x[1];
double stop=exp.x[exp.len-1];

if (sim.x[0]>start) start=sim.x[0];
error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_error_light(struct fitvars *config)
{
printf("get iv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct sim;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/celight.dat");
strcpy(sim_name,"./sim/celight/realce.dat");
strcpy(out_name,"./ce_light_error.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");
//getchar();
//inter_dump(&sim);

inter_log_y(&sim);
inter_log_y(&exp);

double start=exp.x[1];
double stop=exp.x[exp.len-1];
double min;

if (sim.x[0]>start) start=sim.x[0];

min=exp.data[0];

inter_sub(&sim,min);
inter_sub(&exp,min);


max=inter_norm_to_one(&exp);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_tpc_error(struct fitvars *config,int ext)
{
printf("get_tpc_error\n");
char exp_name[40];
char sim_name[40];
char config_name[40];
char out_name[40];
char temp[100];
sprintf(exp_name,"./exp/tpc%d/tpc.dat",ext);
sprintf(sim_name,"./sim/tpc%d/tpcfit.dat",ext);
sprintf(out_name,"./tpc_error%d.dat",ext);
sprintf(config_name,"./exp/tpc%d/tpcfit.inp",ext);

double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct simtime;
double start=-6.7;
double stop=-4;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

inter_chop(&exp,1e-7, 1.0);
inter_chop(&simtime,1e-7, 1.0);

FILE *file_config;
if ((file_config=fopen(config_name,"r"))==NULL)
{
    printf("Error opening file: %s.\n", strerror(errno));
    exit (0);
}

fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(start));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(stop));


fclose(file_config);

double min=inter_get_min(&simtime);

inter_sub(&simtime,min);
inter_sub(&exp,min);

inter_purge_zero(&simtime);
inter_purge_zero(&exp);

inter_log_y(&simtime);
inter_log_y(&exp);
inter_log_x(&simtime);
inter_log_x(&exp);

inter_chop(&exp,start, stop);
inter_chop(&simtime,start, stop);


min=exp.data[exp.len-1];

inter_sub(&simtime,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<simtime.x[0]) start=simtime.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&simtime,1.0, 1.0/max);

//inter_save(&exp,"./a.dat");
//inter_save(&simtime,"./b.dat");

if (ext>1)
{
	error=inter_get_diff(&exp,&simtime,out_name,start,stop,config->tpc_light_error_mull);
}else
{
	error=inter_get_diff(&exp,&simtime,out_name,start,stop,config->tpc_dark_error_mull);
}
inter_free(&exp);
inter_free(&simtime);
//exit(0);
printf("get_tpc_error ret= %d %e %e %e\n",ext,error,min,max );
return error;
}

double get_iv_error(struct fitvars *config)
{
printf("get_iv_error\n");
char expfile[1000];
char simfile[1000];
char out_name[1000];
strcpy(expfile,"./exp/jv/jv.dat");
strcpy(simfile,"./sim/jv/ivexternal.dat");
sprintf(out_name,"./iv_error_dark.dat");

double error=0.0;
struct istruct exp;
struct istruct sim;
inter_load_a(&exp,expfile,0,1);
inter_load_a(&sim,simfile,0,1);

double max=0.0;

double start=0.0;

if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];

double stop=0.8;

inter_log_y(&sim);
inter_log_y(&exp);

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



error=inter_get_diff(&exp,&sim,out_name,start,stop,config->ivdark_fitmul);

inter_free(&exp);
inter_free(&sim);
return error;
}

double get_band_error(struct fitvars *config)
{
int dostype=read_value("dos.inp",0,2);
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
if (dostype==0)
{
	Ee=read_value("dos.inp",0,38);
	Eh=read_value("dos.inp",0,40);
	ret=1+tanh( (Ee-Eh)*20.0/Ee);
	ret*=config->banderror_fitmul;
}else
{

	Ee=read_value("dos.inp",0,12);
	Eh=read_value("dos.inp",0,14);
	ret=1+tanh( (Ee-Eh)*10.0/Ee);
	ret*=config->banderror_fitmul;
}

return ret;
}
