// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>



static double Jnl=0.0;
static double Jnr=0.0;
static double Jpl=0.0;
static double Jpr=0.0;

static double Dnl=0.0;
static double Dnc=0.0;
static double Dnr=0.0;
static double Dpl=0.0;
static double Dpc=0.0;
static double Dpr=0.0;

static double nl=0.0;
static double nc=0.0;
static double nr=0.0;

static double pl=0.0;
static double pc=0.0;
static double pr=0.0;

static double xil=0.0;
static double xir=0.0;
static double xipl=0.0;
static double xipr=0.0;

static double dJpdxipl=0.0;
static double dJpdxipc=0.0;
static double dJpdxipr=0.0;

static double dnl=0.0;
static double dnc=0.0;
static double dnr=0.0;

static double dpl=0.0;
static double dpc=0.0;
static double dpr=0.0;

static double munl=0.0;
static double munc=0.0;
static double munr=0.0;

static double mupl=0.0;
static double mupc=0.0;
static double mupr=0.0;


static double wnl=0.0;
static double wnc=0.0;
static double wnr=0.0;

static double wpl=0.0;
static double wpc=0.0;
static double wpr=0.0;

static double dJdxil=0.0;
static double dJdxic=0.0;
static double dJdxir=0.0;

static double dJdphil=0.0;
static double dJdphic=0.0;
static double dJdphir=0.0;

static double dJpdphil=0.0;
static double dJpdphic=0.0;
static double dJpdphir=0.0;


static double dphidxic=0.0;
static double dphidxipc=0.0;

static double *dntrap=NULL;
static double *dntrapdntrap=NULL;
static double *dntrapdn=NULL;
static double *dntrapdp=NULL;
static double *dJdtrapn=NULL;
static double *dJpdtrapn=NULL;

static double *dptrapdp=NULL;
static double *dptrapdptrap=NULL;
static double *dptrap=NULL;
static double *dptrapdn=NULL;
static double *dJpdtrapp=NULL;
static double *dJdtrapp=NULL;
static double *dphidntrap=NULL;
static double *dphidptrap=NULL;
static double *ntlast=NULL;
static double *ptlast=NULL;
void quick_update();

void update_arrays(struct device *in)
{

int i;
int band=0;

	for (i=0;i<in->ymeshpoints;i++)
	{
		in->Fn[i]=in->x[i]-in->phi[i];
		in->Fp[i]=-in->xp[i]-in->phi[i];


		//printf("%e\n",in->Fpt[i]);

		in->Ec[i]=-in->phi[i]-in->Xi[i];
		in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];


		in->dn[i]=get_dn_den(in->x[i]+in->t[i],in->Te[i],in->me[i],in->dostype[i]);
		in->n[i]=get_n_den(in->x[i]+in->t[i],in->Te[i],in->me[i],in->dostype[i]);
		in->dp[i]=get_dp_den(in->xp[i]-in->tp[i],in->Th[i],in->mh[i],in->dostype[i]);
		in->p[i]=get_p_den(in->xp[i]-in->tp[i],in->Th[i],in->mh[i],in->dostype[i]);

		in->wn[i]=get_n_w(in->x[i]+in->t[i],in->Te[i],in->me[i],in->dostype[i]);
		in->wp[i]=get_p_w(in->xp[i]-in->tp[i],in->Th[i],in->mh[i],in->dostype[i]);

		in->mun[i]=get_n_mu();
		in->mup[i]=get_p_mu();

		if (in->ntrapnewton)
		{
			in->nt_all[i]=0.0;
			for (band=0;band<in->srh_bands;band++)
			{
				in->Fnt[i][band]=in->xt[i][band]-in->phi[i];

				in->srh_n_r1[i][band]=get_n_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_1);
				in->srh_n_r2[i][band]=get_n_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_2);
				in->srh_n_r3[i][band]=get_n_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_3);
				in->srh_n_r4[i][band]=get_n_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_4);
				in->dsrh_n_r1[i][band]=get_dn_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_1);
				in->dsrh_n_r2[i][band]=get_dn_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_2);
				in->dsrh_n_r3[i][band]=get_dn_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_3);
				in->dsrh_n_r4[i][band]=get_dn_srh(in->xt[i][band]+in->tt[i],in->Te[i],band,srh_4);

				in->nt[i][band]=get_n_pop_srh(in->xt[i][band]+in->tt[i],in->Te[i],band);
				in->dnt[i][band]=get_dn_pop_srh(in->xt[i][band]+in->tt[i],in->Te[i],band);
				in->nt_all[i]+=in->nt[i][band];

			}
		}

		if (in->ptrapnewton)
		{
			in->pt_all[i]=0.0;
			for (band=0;band<in->srh_bands;band++)
			{
				in->Fpt[i][band]=-in->xpt[i][band]-in->phi[i];

				in->srh_p_r1[i][band]=get_p_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_1);
				in->srh_p_r2[i][band]=get_p_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_2);
				in->srh_p_r3[i][band]=get_p_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_3);
				in->srh_p_r4[i][band]=get_p_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_4);
				in->dsrh_p_r1[i][band]=get_dp_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_1);
				in->dsrh_p_r2[i][band]=get_dp_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_2);
				in->dsrh_p_r3[i][band]=get_dp_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_3);
				in->dsrh_p_r4[i][band]=get_dp_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band,srh_4);

				in->pt[i][band]=get_p_pop_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band);
				in->dpt[i][band]=get_dp_pop_srh(in->xpt[i][band]-in->tpt[i],in->Th[i],band);
				in->pt_all[i]+=in->pt[i][band];
				//printf("%le\n",in->pt[i][band]);
			}
		}

		}


}

void update_solver_vars(struct device *in,int clamp)
{

int i;
int band=0;



double update=0.0;
	for (i=0;i<in->ymeshpoints;i++)
	{

		update=in->b[i];
		if (clamp==TRUE)
		{
			in->phi[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
		}else
		{
			in->phi[i]+=update;

		}

		update=(in->b[in->ymeshpoints*(1)+i]);
		if (clamp==TRUE)
		{
			in->x[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
		}else
		{
			in->x[i]+=update;
		}


		update=(in->b[in->ymeshpoints*(1+1)+i]);
		if (clamp==TRUE)
		{
			in->xp[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));//
		}else
		{
			in->xp[i]+=update;

		}


		if (in->ntrapnewton==TRUE)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				update=(in->b[in->ymeshpoints*(1+1+1+band)+i]);
				if (clamp==TRUE)
				{
					in->xt[i][band]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));

				}else
				{
					in->xt[i][band]+=update;
				}
			}
		}

		if (in->ptrapnewton==TRUE)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				update=(in->b[in->ymeshpoints*(1+1+1+in->srh_bands+band)+i]);
				if (clamp==TRUE)
				{
					in->xpt[i][band]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
				}else
				{
					in->xpt[i][band]+=update;

				}
			}
		}


		}

update_arrays(in);

}

void fill_matrix(struct device *in)
{
//double offset=-0.5;
int band=0;
//double n0=get_n_den(offset,300.0,0.9,1);
//double p0=get_p_den(offset,300.0,0.9,1);
//printf("%le %le %le %le %le %le\n",n0,p0,n0*get_n_srh(offset,300.0,band,srh_1),get_n_srh(offset,300.0,band,srh_2),p0*get_n_srh(offset,300.0,band,srh_3),get_n_srh(offset,300.0,band,srh_4));
//getchar();

update_arrays(in);
//FILE *file_j =fopen("myj.dat","w");
//getchar();
double phil;
double phic;
double phir;
double yl;
double yc;
double yr;
double dyl;
double dyr;
double ddh;
double dh;
int pos=0;

double Ecl=0.0;
double Ecr=0.0;
double Ecc=0.0;
double Evl=0.0;
double Evr=0.0;
double Evc=0.0;

double Tll=300.0;
double Tlc=300.0;
double Tlr=300.0;

double Tel=300.0;
double Tec=300.0;
double Ter=300.0;

double Thl=300.0;
double Thc=300.0;
double Thr=300.0;

double xnr;
double tnr;
double xnl;
double tnl;

double xpr;
double tpr;
double xpl;
double tpl;


double exl;
double exr;
double exc;
double Dexl;
double Dexc;
double Dexr;
double R;

double epr;
double epc;
double epl;

double G;
double Gex;
int i;
double dJdxipc=0.0;
double dJpdxic=0.0;

double mel=0.0;
double mec=0.0;
double mer=0.0;

double mhl=0.0;
double mhc=0.0;
double mhr=0.0;


double e0=0.0;
double e1=0.0;

double dphil=0.0;
double dphic=0.0;
double dphir=0.0;
double deriv=0.0;

double nlast=0.0;
double plast=0.0;
double dt=0.0;

double kll=0.0;
double klc=0.0;
double klr=0.0;

double kl0=0.0;
double kl1=0.0;

double one;

double Rtrapn=0.0;
double Rtrapp=0.0;
double dptrapdnfree=0.0;
double dptrapdntrap=0.0;
double dntrapdpfree=0.0;


		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{

			if (i==0)
			{
				exl=0.0;
				Dexl=in->Dex[0];
				phil=in->Vapplied;
				yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);
				mel=in->me[0];
				mhl=in->mh[0];
				Tll=in->Tll;
				Tel=in->Tll;
				Thl=in->Tll;




				Ecl=-in->Xi[0]-phil;
				Evl=-in->Xi[0]-phil-in->Eg[0];
				epl=in->epsilonr[0]*epsilon0;


				xnl=in->Fi[0];
				tnl=in->Xi[0];
				if (in->interfaceleft==interface_schottky)
				{
					one=in->phibleft-in->Eg[0];
					//printf("%e",in->phibleft);
					//getchar();
				}else
				{
					one=xnl+tnl;
				}

				nl=get_n_den(one,Tel,mel,in->bdos);
				dnl=get_dn_den(one,Tel,mel,in->bdos);
				wnl=get_n_w(one,Tel,mel,in->bdos);


				munl=in->mun[0];




				xpl=-in->Fi[0];
				tpl=in->Xi[0]+in->Eg[0];

				if (in->interfaceleft==interface_schottky)
				{
					one=-in->phibleft;
				}else
				{
					one=xpl-tpl;
				}

				pl=get_p_den(one,Thl,mhl,in->bdos);
				dpl=get_dp_den(one,Thl,mhl,in->bdos);
				wpl=get_p_w(one,Thl,mhl,in->bdos);

				mupl=in->mup[0];

				//printf("left n= %e p= %e \n",nl,pl);


				kll=in->kl[i];

			}else
			{
				Dexl=in->Dex[i-1];
				exl=in->ex[i-1];
				phil=in->phi[i-1];
				yl=in->ymesh[i-1];
				Tll=in->Tl[i-1];
				Tel=in->Te[i-1];
				Thl=in->Th[i-1];
				Ecl=in->Ec[i-1];
				Evl=in->Ev[i-1];


				mel=in->me[i-1];
				mhl=in->mh[i-1];

				nl=in->n[i-1];
				dnl=in->dn[i-1];


				wnl=in->wn[i-1];
				wpl=in->wp[i-1];

				pl=in->p[i-1];
				dpl=in->dp[i-1];
				munl=in->mun[i-1];
				mupl=in->mup[i-1];


				epl=in->epsilonr[i-1]*epsilon0;


				kll=in->kl[i-1];
			}

			Ecc=-in->Xi[i]-in->phi[i];
			Evc=-in->Xi[i]-in->phi[i]-in->Eg[i];

			if (i==(in->ymeshpoints-1))
			{

				Dexr=in->Dex[i];
				exr=0.0;
				phir=in->Vr;
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
				Tlr=in->Tlr;
				Ter=in->Tlr;
				Thr=in->Tlr;

				Ecr=-in->Xi[i]-phir;
				Evr=-in->Xi[i]-phir-in->Eg[i];

				mer=in->me[i];
				mhr=in->mh[i];

				xnr=phir+in->Fi[i];
				tnr=in->Xi[i];
				if (in->interfaceright==interface_schottky)
				{
					one=-in->phibright;
				}else
				{
					one=xnr+tnr;
				}


				nr=get_n_den(one,Ter,mer,in->bdos);
				dnr=get_dn_den(one,Ter,mer,in->bdos);
				wnr=get_n_w(one,Ter,mer,in->bdos);


				xpr=-(phir+in->Fi[i]);
				tpr=in->Xi[i]+in->Eg[i];
				if (in->interfaceright==interface_schottky)
				{
					one=in->phibright-in->Eg[i];
				}else
				{
					one=xpr-tpr;
					//one/=2.0;
				}


				pr=get_p_den(one,Thr,mhr,in->bdos);
				dpr=get_dp_den(one,Thr,mhr,in->bdos);
				wpr=get_p_w(one,Thr,mhr,in->bdos);

				munr=in->mun[i];
				mupr=in->mup[i];

				//printf("right n= %e p= %e \n",nr,pr);

				epr=in->epsilonr[i]*epsilon0;
				klr=in->kl[i];

			}else
			{
				mer=in->me[i+1];
				mhr=in->mh[i+1];

				Dexr=in->Dex[i+1];
				exr=in->ex[i+1];
				phir=in->phi[i+1];
				yr=in->ymesh[i+1];
				Tlr=in->Tl[i+1];
				Ter=in->Te[i+1];
				Thr=in->Th[i+1];

				Ecr=in->Ec[i+1];
				Evr=in->Ev[i+1];


				nr=in->n[i+1];
				dnr=in->dn[i+1];

				wnr=in->wn[i+1];
				wpr=in->wp[i+1];

				pr=in->p[i+1];
				dpr=in->dp[i+1];
				munr=in->mun[i+1];
				mupr=in->mup[i+1];

				epr=in->epsilonr[i+1]*epsilon0;
				klr=in->kl[i+1];

			}


			mec=in->me[i];
			mhc=in->mh[i];

			epc=in->epsilonr[i]*epsilon0;


			exc=in->ex[i];
			Dexc=in->Dex[i];
			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			ddh=(dyl+dyr)/2.0;
			dh=(dyl+dyr);
			phic=in->phi[i];
			Tlc=in->Tl[i];
			Tec=in->Te[i];
			Thc=in->Th[i];

			//FILE *rod=fopen("./test.dat","w");
			/*double r;
			for (r=-1;r<0;r+=1e-4)
			{
			fprintf(rod,"%e %e %e %e %e\n",r,get_n_w(r,Thr,mhr,in->bdos),q*(3.0/2.0)*get_n_den(r,Thr,mhr,in->bdos,dos_all)/get_dn_den(r,Thr,mhr,in->bdos,dos_all),get_dn_w(r,Thr,mhr,in->bdos),q*(3.0/2.0)*(get_n_den(r+1e-4,Thr,mhr,in->bdos,dos_all)/get_dn_den(r+1e-4,Thr,mhr,in->bdos,dos_all)-get_n_den(r-1e-4,Thr,mhr,in->bdos,dos_all)/get_dn_den(r-1e-4,Thr,mhr,in->bdos,dos_all))/2e-4 );
			}
			fclose(rod);
exit(0);*/


				munc=in->mun[i];
				mupc=in->mup[i];

				wnc=in->wn[i];
				wpc=in->wp[i];

				Dnl=munl*(2.0/3.0)*wnl/q;
				Dpl=mupl*(2.0/3.0)*wpl/q;

				Dnc=munc*(2.0/3.0)*wnc/q;
				Dpc=mupc*(2.0/3.0)*wpc/q;

				Dnr=munr*(2.0/3.0)*wnr/q;
				Dpr=mupr*(2.0/3.0)*wpr/q;


				Dnl=(Dnl+Dnc)/2.0;
				Dnr=(Dnr+Dnc)/2.0;

				Dpl=(Dpl+Dpc)/2.0;
				Dpr=(Dpr+Dpc)/2.0;

				munl=(munl+munc)/2.0;
				munr=(munr+munc)/2.0;

				mupl=(mupl+mupc)/2.0;
				mupr=(mupr+mupc)/2.0;



				nc=in->n[i];
				pc=in->p[i];

				dnc=in->dn[i];
				dpc=in->dp[i];


			klc=in->kl[i];
			nlast=in->nlast[i];
			plast=in->plast[i];

			for (band=0;band<in->srh_bands;band++)
			{
				ntlast[band]=in->ntlast[i][band];
				ptlast[band]=in->ptlast[i][band];
			}

			dt=in->dt;

	R=in->R[i];
	Gex=in->Gex[i];

	e0=(epl+epc)/2.0;
	e1=(epc+epr)/2.0;

	kl0=(klc+kll)/2.0;
	kl1=(klr+klc)/2.0;

	dphil=-e0/dyl/ddh;
	dphic=e0/dyl/ddh+e1/dyr/ddh;
	dphir=-e1/dyr/ddh;

	deriv=phil*dphil+phic*dphic+phir*dphir;


		dphidxic=q*(dnc);
		dphidxipc=-q*(dpc);

		if (in->ntrapnewton==TRUE)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				dphidntrap[band]=q*(in->dnt[i][band]);
			}
		}

		if (in->ptrapnewton==TRUE)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				dphidptrap[band]=-q*(in->dpt[i][band]);
				//dphidxipc+=-q*(in->dpt[i]);
			}
		}



	G=in->G[i];



			xil=q*2.0*(3.0/2.0)*(Ecc-Ecl)/((wnc+wnl));
			xir=q*2.0*(3.0/2.0)*(Ecr-Ecc)/((wnr+wnc));

			//double dxil=-q*2.0*(3.0/2.0)*(Ecc-Ecl)/pow((wnc+wnl),2.0);
			//double dxir=-q*2.0*(3.0/2.0)*(Ecr-Ecc)/pow((wnr+wnc),2.0);

			xipl=q*2.0*(3.0/2.0)*(Evc-Evl)/(wpc+wpl);
			xipr=q*2.0*(3.0/2.0)*(Evr-Evc)/(wpr+wpc);

			//printf("%e %e %e\n",q*2.0*(3.0/2.0)*(Ecc-Ecl)/((wnc+wnl)),2.0*(3.0/2.0)*(mec-mel)/(mec+mel),2.0*(wnc-wnl)/(wnc+wnl));
			//getchar();



			Jnl=(Dnl/dyl)*(B(-xil)*nc-B(xil)*nl);
			Jnr=(Dnr/dyr)*(B(-xir)*nr-B(xir)*nc);




			Jpl=(Dpl/dyl)*(B(-xipl)*pl-B(xipl)*pc);
			Jpr=(Dpr/dyr)*(B(-xipr)*pc-B(xipr)*pr);

			if (i==0)
			{
				if (in->interfaceleft==TRUE)
				{
					Jnl=in->vl*(nc-nl);
				}
				in->Jnleft=Jnl;
				in->Jpleft=Jpl;
			}

			if (i==in->ymeshpoints-1)
			{
				in->Jnright=Jnr;
				in->Jpright=Jpr;
			}
//fprintf(file_j,"%e %e\n",in->ymesh[i],-(get_top_from_p(pc,in->Te,in->mh,in->Nv,in->dostype[i],dos_all)+in->Xi[i]+in->Eg[i]+in->phi[i]));
//fprintf(file_j,"%e %e\n",in->ymesh[i]-dyl/2.0,Jnl);
//fprintf(file_j,"%e %e\n",in->ymesh[i]+dyl/2.0,Jnr);
//fprintf(file_j,"%e %e %e\n",in->ymesh[i]-dyl/2.0,Jnl,Jpl);
//fprintf(file_j,"%e %e %e\n",in->ymesh[i]+dyr/2.0,Jnr,Jpr);

			in->Jn[i]=q*(Jnl+Jnr)/2.0;
			in->Jp[i]=q*(Jpl+Jpr)/2.0;
			//double jntest=munc*nc*(Ecr-Ecl)/ddh+kb*in->Te[i]*munc*(nr-nl)/ddh;
//fprintf(file_j,"%e %e %e\n",in->ymesh[i]-dyl/2.0,Jnl,jntest);
//fprintf(file_j,"%e %e\n",in->ymesh[i]+dyl/2.0,Jnr);
			dJdxil=(Dnl/dyl/ddh)*(B(xil)*dnl);
			dJdxic=-(Dnl/dyl)*B(-xil)*dnc/ddh-(Dnr/dyr)*B(xir)*dnc/ddh;
			dJdxir=(Dnr/dyr)*B(-xir)*dnr/ddh;

			if (i==0)
			{
				if (in->interfaceleft==TRUE)
				{
				dJdxil=0.0;
				dJdxic=-(1.0/dyl)*in->vl*2.0*dnc/ddh-(Dnr/dyr)*B(xir)*dnc/ddh;
					//Jnl=in->vl*(nc-nl);
				}
			}


			dJpdxipl=-(Dpl/dyl/ddh)*(B(-xipl)*dpl);
			dJpdxipc=(Dpl/dyl/ddh)*B(xipl)*dpc+(Dpr/dyr/ddh)*B(-xipr)*dpc;
			dJpdxipr=-(Dpr/dyr/ddh)*B(xipr)*dpr;



			dJdphil=(munl/dyl/ddh)*(dB(-xil)*nc+dB(xil)*nl);
			dJdphic=-((munl/dyl)*(dB(-xil)*nc+dB(xil)*nl)/ddh+(munr/dyr)*(dB(-xir)*nr+dB(xir)*nc)/ddh);
			dJdphir=(munr/dyr/ddh)*(dB(-xir)*nr+dB(xir)*nc);

			if (i==0)
			{
				if (in->interfaceleft==TRUE)
				{
				dJdphil=0.0;
				dJdxic=-((munr/dyr)*(dB(-xir)*nr+dB(xir)*nc)/ddh);
				}
			}

			dJpdphil=((mupl)/dyl/ddh)*(dB(-xipl)*pl+dB(xipl)*pc);
			dJpdphic=-(((mupl)/dyl/ddh)*(dB(-xipl)*pl+dB(xipl)*pc)+(mupr/dyr/ddh)*(dB(-xipr)*pc+dB(xipr)*pr));
			dJpdphir=(mupr/dyr/ddh)*(dB(-xipr)*pc+dB(xipr)*pr);



			if (in->go_time==TRUE)
			{
				dJdxic+=-dnc/dt;
			}

			if (in->go_time==TRUE)
			{
				dJpdxipc+=dpc/dt;
			}




			dJdxipc=0.0;
			dJpdxic=0.0;
			Rtrapn=0.0;
			Rtrapp=0.0;

			in->nrelax[i]=0.0;
			in->ntrap_to_p[i]=0.0;
			in->prelax[i]=0.0;
			in->ptrap_to_n[i]=0.0;

			if (in->ntrapnewton==TRUE)
			{
				for (band=0;band<in->srh_bands;band++)
				{
					dJdtrapn[band]=0.0;
					dJpdtrapn[band]=0.0;
					dntrap[band]=nc*in->srh_n_r1[i][band]-in->srh_n_r2[i][band]-pc*in->srh_n_r3[i][band]+in->srh_n_r4[i][band];
					dntrapdntrap[band]=nc*in->dsrh_n_r1[i][band]-in->dsrh_n_r2[i][band]-pc*in->dsrh_n_r3[i][band]+in->dsrh_n_r4[i][band];
					dntrapdn[band]=dnc*in->srh_n_r1[i][band];
					dntrapdp[band]=-dpc*in->srh_n_r3[i][band];
					Rtrapn+=nc*in->srh_n_r1[i][band]-in->srh_n_r2[i][band];
					dJdxic-=dnc*in->srh_n_r1[i][band];
					dJdtrapn[band]-=nc*in->dsrh_n_r1[i][band]-in->dsrh_n_r2[i][band];
					Rtrapp+=-(-pc*in->srh_n_r3[i][band]+in->srh_n_r4[i][band]);
					dJpdxipc+=-(-dpc*in->srh_n_r3[i][band]);
					dJpdtrapn[band]=-(-pc*in->dsrh_n_r3[i][band]+in->dsrh_n_r4[i][band]);
					if (in->go_time==TRUE)
					{
						dntrap[band]+=-(in->nt[i][band]-ntlast[band])/dt;
						dntrapdntrap[band]+=-(in->dnt[i][band])/dt;
					}

					in->nrelax[i]+=nc*in->srh_n_r1[i][band]-in->srh_n_r2[i][band];
					in->ntrap_to_p[i]+=-(-pc*in->srh_n_r3[i][band]+in->srh_n_r4[i][band]);


				}
			}

			//band=0;
			//printf("heren %le %le %le %le\n",in->Vapplied,nc*in->srh_n_r1[i][band]-in->srh_n_r2[i][band]-pc*in->srh_n_r3[i][band]+in->srh_n_r4[i][band],nc*in->srh_n_r1[i][band]-in->srh_n_r2[i][band],-pc*in->srh_n_r3[i][band]+in->srh_n_r4[i][band]);

			if (in->ptrapnewton==TRUE)
			{

				for (band=0;band<in->srh_bands;band++)
				{
					//dJdtrapn[band]=0.0;
					dJpdtrapp[band]=0.0;
					dJdtrapp[band]=0.0;
					dptrap[band]=pc*in->srh_p_r1[i][band]-in->srh_p_r2[i][band]-nc*in->srh_p_r3[i][band]+in->srh_p_r4[i][band];
					dptrapdptrap[band]=pc*in->dsrh_p_r1[i][band]-in->dsrh_p_r2[i][band]-nc*in->dsrh_p_r3[i][band]+in->dsrh_p_r4[i][band];
					dptrapdp[band]=dpc*in->srh_p_r1[i][band];
					dptrapdn[band]=-dnc*in->srh_p_r3[i][band];

					Rtrapp+=pc*in->srh_p_r1[i][band]-in->srh_p_r2[i][band];
					dJpdxipc+=dpc*in->srh_p_r1[i][band];
					dJpdtrapp[band]+=pc*in->dsrh_p_r1[i][band]-in->dsrh_p_r2[i][band];

					Rtrapn+=-(-nc*in->srh_p_r3[i][band]+in->srh_p_r4[i][band]);
					dJdxic-=-(-dnc*in->srh_p_r3[i][band]);
					dJdtrapp[band]-=-(-nc*in->dsrh_p_r3[i][band]+in->dsrh_p_r4[i][band]);

					if (in->go_time==TRUE)
					{
						dptrap[band]+=-(in->pt[i][band]-ptlast[band])/dt;
						dptrapdptrap[band]+=-(in->dpt[i][band])/dt;
					}

					in->prelax[i]+=pc*in->srh_p_r1[i][band]-in->srh_p_r2[i][band];
					in->ptrap_to_n[i]+=-(-nc*in->srh_p_r3[i][band]+in->srh_p_r4[i][band]);
				}

			}

			//band=0;
			//printf("hereh %le %le %le %le\n",in->Vapplied,pc*in->srh_p_r1[i][band]-in->srh_p_r2[i][band]-nc*in->srh_p_r3[i][band]+in->srh_p_r4[i][band],pc*in->srh_p_r1[i][band]-in->srh_p_r2[i][band],-nc*in->srh_p_r3[i][band]+in->srh_p_r4[i][band]);


			in->Rn[i]=Rtrapn;
			in->Rp[i]=Rtrapp;
			//Rtrapp=1e24;
			//Rtrapn=1e24;




			if (i!=0)
			{
				in->Ti[pos]=i;
				in->Tj[pos]=i-1;
				in->Tx[pos]=dphil;
				pos++;
				//electron

					in->Ti[pos]=in->ymeshpoints*(1)+i;
					in->Tj[pos]=in->ymeshpoints*(1)+i-1;
					in->Tx[pos]=dJdxil;
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1)+i;
					in->Tj[pos]=i-1;
					in->Tx[pos]=dJdphil;
					pos++;

					//hole
					in->Ti[pos]=in->ymeshpoints*(1+1)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1)+i-1;
					in->Tx[pos]=dJpdxipl;
					pos++;

					in->Ti[pos]=i+in->ymeshpoints*(1+1);
					in->Tj[pos]=i-1;
					in->Tx[pos]=dJpdphil;
					pos++;




			}

			//phi
			in->Ti[pos]=i;
			in->Tj[pos]=i;
			in->Tx[pos]=dphic;
			pos++;

			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints*(1);
			in->Tx[pos]=dphidxic;
			pos++;

			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints*(1+1);
			in->Tx[pos]=dphidxipc;
			pos++;


			//electron

			in->Ti[pos]=in->ymeshpoints*(1)+i;
			in->Tj[pos]=in->ymeshpoints*(1)+i;
			in->Tx[pos]=dJdxic;
			pos++;


			in->Ti[pos]=in->ymeshpoints*(1)+i;
			in->Tj[pos]=i;
			in->Tx[pos]=dJdphic;
			pos++;





			//hole
			in->Ti[pos]=in->ymeshpoints*(1+1)+i;
			in->Tj[pos]=in->ymeshpoints*(1+1)+i;
			in->Tx[pos]=dJpdxipc;
			pos++;


			in->Ti[pos]=in->ymeshpoints*(1+1)+i;
			in->Tj[pos]=i;
			in->Tx[pos]=dJpdphic;
			pos++;



			if (in->ntrapnewton==TRUE)
			{
				for (band=0;band<in->srh_bands;band++)
				{
					in->Ti[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tx[pos]=dntrapdntrap[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tj[pos]=in->ymeshpoints*1+i;
					in->Tx[pos]=dntrapdn[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1)+i;
					in->Tx[pos]=dntrapdp[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tx[pos]=dJdtrapn[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+1)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tx[pos]=dJpdtrapn[band];
					pos++;

					in->Ti[pos]=i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+band)+i;
					in->Tx[pos]=dphidntrap[band];
					pos++;


				}
				/*



				in->Ti[pos]=in->ymeshpoints*(1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+1)+i;
				in->Tx[pos]=dJdxipc;
				pos++;

*/

			}

			if (in->ptrapnewton==TRUE)
			{
				for (band=0;band<in->srh_bands;band++)
				{
					in->Ti[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tx[pos]=dptrapdptrap[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1)+i;
					in->Tx[pos]=dptrapdp[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1)+i;
					in->Tx[pos]=dptrapdn[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+1)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tx[pos]=dJpdtrapp[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1)+i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tx[pos]=dJdtrapp[band];
					pos++;

					in->Ti[pos]=i;
					in->Tj[pos]=in->ymeshpoints*(1+1+1+in->srh_bands+band)+i;
					in->Tx[pos]=dphidptrap[band];
					pos++;
				}

				/*

				in->Ti[pos]=in->ymeshpoints*(1+1+1+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1)+i;
				in->Tx[pos]=dptrapdnfree;
				pos++;

				in->Ti[pos]=in->ymeshpoints*(1+1+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+1)+i;
				in->Tx[pos]=dntrapdpfree;
				pos++;*/

			}

			if (i!=(in->ymeshpoints-1))
			{
				//phi
				in->Ti[pos]=i;
				in->Tj[pos]=i+1;
				in->Tx[pos]=dphir;
				pos++;




				//electron
				in->Ti[pos]=in->ymeshpoints*(1)+i;
				in->Tj[pos]=in->ymeshpoints*(1)+i+1;
				in->Tx[pos]=dJdxir;
				pos++;

				in->Ti[pos]=i+in->ymeshpoints*(1);
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJdphir;
				pos++;

				//hole
				in->Ti[pos]=in->ymeshpoints*(1+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+1)+i+1;
				in->Tx[pos]=dJpdxipr;
				pos++;

				in->Ti[pos]=i+in->ymeshpoints*(1+1);
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJpdphir;
				pos++;




			}

			//Possion
			in->b[i]=-(deriv);

			in->b[i]+=-(-q*(in->p[i]-in->n[i]));

			for (band=0;band<in->srh_bands;band++)
			{
				in->b[i]+=-(-q*(in->pt[i][band]-in->nt[i][band]));
			}

			in->b[i]+=-(-q*in->Nad[i]);

			in->b[in->ymeshpoints*(1)+i]=-((Jnr-Jnl)/ddh-Rtrapn);

			if (in->go_time==TRUE)
			{
				in->b[in->ymeshpoints*(1)+i]-=-(nc-nlast)/dt;
			}

			if (in->Dphoton==TRUE) in->b[in->ymeshpoints*(1)+i]-=in->Gex[i];
			//hole
			in->b[in->ymeshpoints*(1+1)+i]=-((Jpr-Jpl)/ddh+Rtrapp);
			//printf("%le %le\n",Rtrapn,Rtrapp);

			if (in->Dphoton==TRUE) in->b[in->ymeshpoints*(1+1)+i]-=-in->Gex[i];

			if (in->go_time==TRUE)
			{
				in->b[in->ymeshpoints*(1+1)+i]-=(pc-plast)/dt;
			}

			if (in->ntrapnewton==TRUE)
			{
				for (band=0;band<in->srh_bands;band++)
				{
					in->b[in->ymeshpoints*(1+1+1+band)+i]=-(dntrap[band]);
				}
			}

			if (in->ptrapnewton==TRUE)
			{
				for (band=0;band<in->srh_bands;band++)
				{
					in->b[in->ymeshpoints*(1+1+1+in->srh_bands+band)+i]=-(dptrap[band]);
				}

			}

		if (pos>in->N)
		{
			printf("Error\n");
		}
		}



//	solver_dump_matrix(in->M,pos,in->Ti,in->Tj, in->Tx,in->b);
//fclose(file_j);
//printf("Check J file\n");
//getchar();

}

double get_cur_error(struct device *in)
{
int i;
double phi=0.0;
double n=0.0;
double p=0.0;
double x=0.0;
double te=0.0;
double th=0.0;
double tl=0.0;
double ttn=0.0;
double ttp=0.0;
int band=0;
for (i=0;i<in->ymeshpoints;i++)
{
		phi+=mod(in->b[i]);

		n+=mod(in->b[in->ymeshpoints*(1)+i]);
		p+=+mod(in->b[in->ymeshpoints*(1+1)+i]);

		if (in->ntrapnewton==TRUE)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				ttn+=mod(in->b[in->ymeshpoints*(1+1+1+band)+i]);
				//printf("error %le\n",mod(in->b[in->ymeshpoints*(1+1+1+band)+i]));
			}
		}

		if (in->ptrapnewton==TRUE)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				ttp+=mod(in->b[in->ymeshpoints*(1+1+1+in->srh_bands+band)+i]);
			}
		}


}

double tot=phi+n+p+x+te+th+tl+ttn+ttp;
//printf("%e %e %e %e %e %e %e %e %e\n",phi,n,p,x,te,th,tl,ttn,ttp);
if (isnan( tot))
{
	printf("%e %e %e %e %e %e %e %e %e\n",phi,n,p,x,te,th,tl,ttn,ttp);
	solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
	exit(0);
}

return tot;
}

void solver_get_memory(struct device *in)
{
int i;
in->N=in->ymeshpoints*3-2;	//Possion main

in->N+=in->ymeshpoints*3-2;	//Je main
in->N+=in->ymeshpoints*3-2;	//Jh main

in->N+=in->ymeshpoints*3-2;	//dJe/phi
in->N+=in->ymeshpoints*3-2;	//dJh/phi

in->N+=in->ymeshpoints;		//dphi/dn
in->N+=in->ymeshpoints;		//dphi/dh

//in->N+=in->ymeshpoints;		//dJndp
//in->N+=in->ymeshpoints;		//dJpdn

in->M=in->ymeshpoints;	//Pos

in->M+=in->ymeshpoints;		//Je
in->M+=in->ymeshpoints;		//Jh

if (in->ntrapnewton==TRUE)
{
	for (i=0;i<in->srh_bands;i++)
	{
		in->N+=in->ymeshpoints;		//dntrapdn
		in->N+=in->ymeshpoints;		//dntrapdntrap
		in->N+=in->ymeshpoints;		//dntrapdp
		in->N+=in->ymeshpoints;		//dJndtrapn
		in->N+=in->ymeshpoints;		//dJpdtrapn
		in->N+=in->ymeshpoints;		//dphidntrap

		in->M+=in->ymeshpoints;		//nt
	}

}

if (in->ptrapnewton==TRUE)
{
	for (i=0;i<in->srh_bands;i++)
	{
		in->N+=in->ymeshpoints;		//dptrapdp
		in->N+=in->ymeshpoints;		//dptrapdptrap
		in->N+=in->ymeshpoints;		//dptrapdn
		in->N+=in->ymeshpoints;		//dJpdtrapp
		in->N+=in->ymeshpoints;		//dJdtrapp
		in->N+=in->ymeshpoints;		//dphidptrap

		in->M+=in->ymeshpoints;		//pt
	}
}

in->Ti =    malloc(in->N*sizeof(int));
in->Tj =    malloc(in->N*sizeof(int));
in->Tx = malloc(in->N*sizeof(double));
in->b = malloc(in->M*sizeof(double));
in->back_b = malloc(in->M*sizeof(double));


dntrap=malloc(in->srh_bands*sizeof(double));
dntrapdntrap=malloc(in->srh_bands*sizeof(double));
dntrapdn=malloc(in->srh_bands*sizeof(double));
dntrapdp=malloc(in->srh_bands*sizeof(double));
dJdtrapn=malloc(in->srh_bands*sizeof(double));
dJpdtrapn=malloc(in->srh_bands*sizeof(double));
dphidntrap=malloc(in->srh_bands*sizeof(double));
ntlast=malloc(in->srh_bands*sizeof(double));

dptrapdp=malloc(in->srh_bands*sizeof(double));
dptrapdptrap=malloc(in->srh_bands*sizeof(double));
dptrap=malloc(in->srh_bands*sizeof(double));
dptrapdn=malloc(in->srh_bands*sizeof(double));
dJpdtrapp=malloc(in->srh_bands*sizeof(double));
dJdtrapp=malloc(in->srh_bands*sizeof(double));
dphidptrap=malloc(in->srh_bands*sizeof(double));
ptlast=malloc(in->srh_bands*sizeof(double));
}

void solver_free_memory(struct device *in)
{

free(dntrap);
free(dntrapdntrap);
free(dntrapdn);
free(dntrapdp);
free(dJdtrapn);
free(dJpdtrapn);
free(dphidntrap);
free(ntlast);

free(dptrapdp);
free(dptrapdptrap);
free(dptrap);
free(dptrapdn);
free(dJpdtrapp);
free(dJdtrapp);
free(dphidptrap);
free(ptlast);

free(in->Ti);
free(in->Tj);
free(in->Tx);
free(in->b);
free(in->back_b);

}

void dump_for_plot(struct device *in)
{
#ifdef dump_newton

	int i;
	int band;
	FILE *out =fopena(in->outputpath,"./band.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e %e %e ",in->ymesh[i],in->Ec[i],in->Ev[i],in->Fi[i],in->Fn[i],in->Fp[i]);

			for (band=0;band<in->srh_bands;band++)
			{
				fprintf(out,"%e %e ",in->Fnt[i][band],in->Fpt[i][band]);
			}

			fprintf(out,"\n");
	}

	fclose(out);

	out =fopena(in->outputpath,"./phi.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e\n",in->ymesh[i],in->phi[i]);
	}

	fclose(out);

	out =fopena(in->outputpath,"./j.dat","w");
	double Jtot=0.0;
	fprintf(out,"%e %e %e %e\n",-in->ymesh[0],q*in->Jnleft,q*in->Jpleft,q*(in->Jpleft-in->Jnleft));
	for (i=1;i<in->ymeshpoints-2;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);
		Jtot=0.0;


			fprintf(out,"%e %e %e\n",in->Jn[i],in->Jp[i],(in->Jp[i]-in->Jn[i]));

	}
	fprintf(out,"%e %e %e %e\n",in->ymesh[in->ymeshpoints-1],q*in->Jnright,q*in->Jpright,q*(in->Jpright-in->Jnright));
	fclose(out);

	out =fopena(in->outputpath,"./Fn.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);


		fprintf(out,"%e ",in->Fn[i]);

		fprintf(out,"\n");

	}

	fclose(out);

	out =fopena(in->outputpath,"./Fp.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		fprintf(out,"%e ",in->Fp[i]);

		fprintf(out,"\n");

	}

	fclose(out);


	out =fopena(in->outputpath,"./mu.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		fprintf(out,"%e %e ",in->mun[i],in->mup[i]);

		fprintf(out,"\n");

	}

	fclose(out);

	out =fopena(in->outputpath,"./n.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);
		fprintf(out,"%e %e ",in->n[i],in->p[i]);
		fprintf(out,"\n");
	}

	fclose(out);

	out =fopena(in->outputpath,"./nft.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);
		fprintf(out,"%e %e ",in->n[i]+in->nt_all[i],in->p[i]+in->pt_all[i]);
		fprintf(out,"\n");
	}

	fclose(out);

	out =fopena(in->outputpath,"./trap.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		//for (band=0;band<in->srh_bands;band++)
		//{
			fprintf(out,"%e %e ",in->nt_all[i],in->pt_all[i]);
		//}

		fprintf(out,"\n");
	}

	fclose(out);


	out =fopena(in->outputpath,"./Dex.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e\n",in->ymesh[i],in->ex[i],in->G[i],in->Gex[i]);
	}

	fclose(out);

	out =fopena(in->outputpath,"./R.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e\n",in->ymesh[i],in->Rn[i],in->Rp[i]);
	}
	fclose(out);

	out =fopena(in->outputpath,"./fsrhn.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e ",in->ymesh[i]);
			for (band=0;band<in->srh_bands;band++)
			{
				fprintf(out,"%e %e ",in->Fnt[i][band],-in->phi[i]-in->Xi[i]+dos_srh_get_fermi_n(in->n[i], in->p[i],band));
			}
			fprintf(out,"\n");
	}
	fclose(out);

	out =fopena(in->outputpath,"./fsrhh.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e ",in->ymesh[i]);
			for (band=0;band<in->srh_bands;band++)
			{
				fprintf(out,"%e %e ",in->Fpt[i][band],-in->phi[i]-in->Xi[i]-in->Eg[i]-dos_srh_get_fermi_p(in->n[i], in->p[i],band));
			}
			fprintf(out,"\n");
	}
	fclose(out);


#endif
}

int solve_cur(struct device *in)
{
double error;
int ittr=0;
#ifdef print_newtonerror
dprintf("Solve cur\n");
#endif



int doback=FALSE;
#ifdef only
only_update_thermal=FALSE;
#endif
//in->enable_back=FALSE;
int keep_going=FALSE;
int thermalrun=0;
double check[10];
int cpos=0;
	do
	{

		fill_matrix(in);
//solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
//getchar();

//dump_for_plot(in);
//plot_now(in,"plot");
	//solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
//getchar();

		if (in->enable_back)
		{
			if (ittr<5)
			{
				doback=FALSE;
			}else
			{
				doback=TRUE;
			}
		}
		//doback=FALSE;
		//if (ittr==6) doback=TRUE;

		//if (error<1e-6) doback=TRUE;

		//doback=FALSE;
		//if (ittr<3) doback=FALSE;
		//doback=TRUE;
		//in->enable_back=FALSE;
		if (in->stop==TRUE)
		{
			break;
		}

		//printf("%d %d\n",in->enable_back,doback);
		//getchar();

		if (doback==TRUE)
		{

			do_backtrack(in);
			update_solver_vars(in,FALSE);
		}else
		{
			solver(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
			update_solver_vars(in,TRUE);
			//getchar();
		}

		error=get_cur_error(in);

		//thermalrun++;
		if (thermalrun==40) thermalrun=0;
		//update(in);
//getchar();

		#ifdef print_newtonerror
		dprintf("%d Cur error = %e %e",ittr,error,in->Vapplied);
		if (doback==TRUE)
		{
			dprintf(" b");
		}


		dprintf("\n");
		#endif

		in->last_error=error;
		ittr++;

		#ifdef dump_converge
		in->converge=fopena(in->outputpath,"./converge.dat","a");
		fprintf(in->converge,"%e\n",error);
		fclose(in->converge);
		#endif
		keep_going=FALSE;

		quick_update();

		//printf("%le\n",in->min_cur_error);
		//getchar();
		in->dd_conv=FALSE;
		if ((ittr<2)&&(error<in->min_cur_error)) in->dd_conv=TRUE;
		if ((ittr<in->max_electrical_itt)&&(error>in->min_cur_error)) keep_going=TRUE;
		if (ittr<10) keep_going=TRUE;

		check[cpos]=error;
		cpos++;
		if (cpos>=10)
		{
			cpos=0;
			if ((check[0]<error)||(check[1]<error)) keep_going=FALSE;
		}

	}while(keep_going==TRUE);

//getchar();
		//int r;
		//for (r=0;r<in->ymeshpoints;r++)
		//{
		//in->mun[r]=(get_mat_param_mobility(&(in->mat.l[in->ymatlayers[r]]),mat_mue,in->n[r])-in->mun[r])/100.0;
		//in->mup[r]=(get_mat_param_mobility(&(in->mat.l[in->ymatlayers[r]]),mat_muh,in->p[r])-in->mup[r])/100.0;
		//}
dump_for_plot(in);

plot_now(in,"plot");
//getchar();
in->odes+=in->M;
//getchar();
return 0;
}

